;+
; Project     : HESSI
;
; Name        : RD_GOES_SDAC
;
; Purpose     : read GOES SDAC FITS data (a socket wrapper around GFITS_R)
;
; Category    : synoptic gbo
;
; Syntax      : IDL> rd_goes_sdac
;
; Inputs      : See GFITS_R keywords
;
; Outputs     : See GFITS_R keywords
;
; Keywords    : See GFITS_R keywords
;               STIME, ETIME = start/end times to search
;               REMOTE = force a network search
;
; History     : Written 15 June 2005, D. Zarro, (L-3Com/GSFC)
;               Modified 24 Nov 2005, Zarro (L-3Com/GSFC) 
;                - preserve currently defined $GOES_FITS
;               Modified 26 Dec 2005, Zarro (L-3Com/GSFC) 
;                - support varied input time formats with anytim
;               Modified 30 Dec 2005, Zarro (L-3Com/GSFC)
;                - improved by only downloading required files
;               Modified Election Night 7 Nov 2006, Zarro (ADNET/GSFC) 
;                - check that $GOES_FITS is a valid archive 
;
; Contact     : dzarro@solar.stanford.edu
;-

pro rd_goes_sdac,stime=stime,etime=etime,_ref_extra=extra,remote=remote,$
                 sat=sat

;-- if $GOES_FITS defined, then read files locally

if (not valid_time(stime)) or (not valid_time(etime)) then begin
 pr_syntax,'rd_goes_sdac,stime=stime,etime=time,tarray=time,yarray=data'
 return
endif

tstart=anytim(stime,/ecs) & tend=anytim(etime,/ecs)

if is_number(remote) then remote =  (0b >  byte(remote) < 1b) else remote=0
if not is_number(sat) then sat=12

;-- if not forcing a remote, check if GOES_FITS has an index file. If it does, then
;   we should search it since it is a valid GOES archive. If not, move on 

if not remote then begin
 index=concat_dir('$GOES_FITS','index.dat')
 chk=loc_file(index,count=count)
 if (count eq 1) then begin
  gfits_r,stime=tstart,etime=tend,sat=sat,_extra=extra,/sdac
  return
 endif
endif

;-- determine remote location of files

server=goes_server(network=network,_extra=extra,/sdac,path=path,/full)

;-- Use CATCH if we have problems. 

goes_fits_sav=chklog('$GOES_FITS')
error=0
catch,error
if error ne 0 then begin
 message,err_state(),/cont
 catch,/cancel
 mklog,'GOES_FITS',goes_fits_sav
 return
endif

;-- if currently defined GOES_FITS is writeable, we use it. 
;   Otherwise, create a temporary GOES_FITS directory.

use_temp=0b
chk=write_dir('$GOES_FITS',out=goes_dir)
if (1-chk) then begin
 use_temp=1b
 goes_dir=goes_temp_dir()
 if not is_dir(goes_dir) then mk_dir,goes_dir
 mklog,'GOES_FITS',goes_dir
endif

;-- cycle thru each available GOES satellite until we get a match

sats=goes_sat(/number)

;-- start with user-specified satellite

if is_number(sat[0]) then begin
 chk=where(sat[0] eq sats,count,complement=complement,ncomp=ncomp)
 if ncomp gt 0 then sats=sats[complement]
 if count gt 0 then sats=[sat[0],sats]
endif

goes_url=server+path
http=obj_new('http')
dsat=-1
for i=0,n_elements(sats)-1 do begin

 tsat=sats[i]

;-- determine which file names to copy

 files=goes_fits_files(tstart,tend,_extra=extra,sat=tsat,/no_comp)
 if is_blank(files) then continue

;-- check if they exist at the server, and download

 goes_files=goes_url+'/'+files
 found_sat=0b

;-- if server is down, check last downloaded files
 
 if network then begin
  for k=0,n_elements(files)-1 do begin
   http->copy,goes_files[k],out_dir=goes_dir,_extra=extra,/verb,err=err
   if is_blank(err) then found_sat=1b
  endfor
 endif else found_sat=1b

;-- if found, then read downloaded files 

 if found_sat then begin
  gfits_r,stime=tstart,etime=tend,sat=tsat,_extra=extra,error=error,/sdac,/no_retry
 
;-- if everything is ok, then bail out otherwise try another satellite

  if not error then break
 endif

endfor

obj_destroy,http
mklog,'GOES_FITS',goes_fits_sav

;-- clean up old files 

if use_temp then begin
 old_files=file_since(older=10,patt='go*',count=count,path=goes_dir)
 if count gt 0 then file_delete,old_files,/quiet
endif


return & end
