;+
;PROCEDURE:  themis_init
;PURPOSE:    Initializes system variables for themis.
;
; The system variable !THEMIS is defined here.  The elements of this structure are explained below:
;
; !THEMIS.LOCAL_DATA_DIR    This is the root location for all THEMIS data files.
;                  The THEMIS software expects all data files to reside in specific subdirectories relative
;                  to this root directory.;
;
; !THEMIS.REMOTE_DATA_DIR   This is the URL of the server that can provide the data files.
;                  (default is: "http://sprg.ssl.berkeley.edu/data/themis/")
;                  if the software does not find a needed file in LOCAL_DATA_DIR, and REMOTE_DATA_DIR is defined,
;                  the software will attempt to download the file from REMOTE_DATA_DIR, place it in LOCAL_DATA_DIR
;                  with the same relative pathname, and then continue processing.
;
; Typical examples:
;
;          Desktop UNIX/LINUX computer located at SSL
;   LOCAL_DATA_DIR  = '/disks/data/themis/'               ; This master directory is read only.
;   REMOTE_DATA_DIR = ''                                  ; Should be empty sting. (/disks/data/themis and server are the same)
;
;          Desktop WINDOWS computer located at SSL
;   LOCAL_DATA_DIR  = '\\justice\data\themis\'            ; Justice is a samba server (physically the same as /disk/data/themis)
;   REMOTE_DATA_DIR = ''
;
;          laptop WINDOWS computer located far from a data server, but with internet connection.
;   LOCAL_DATA_DIR  = 'C;\data\themis\'                              ; Local (portable) directory on laptop
;   REMOTE_DATA_DIR = 'http://sprg.ssl.berkeley.edu/data/themis/'    ;  URL used to download data to LOCAL_DATA_DIR
;
;          MacOS computer located away from SSL without a nearby data server
;   LOCAL_DATA_DIR  = '/data/themis/'                              ; Local (portable) directory on laptop
;   REMOTE_DATA_DIR = 'http://sprg.ssl.berkeley.edu/data/themis/'    ;  URL used to download data to LOCAL_DATA_DIR
;
;   Note: If automatic downloads are used. (i.e. REMOTE_DATA_DIR is not an empty string) the user must ensure that
;   LOCAL_DATA_DIR is writeable.
;
;   Note to WINDOWS users: the WINDOWS version of IDL accepts both the '\' and '/' character as the directory
;   separation character. The converse is not true.
;
;
;
;-
pro themis_init,reset=reset

defsysv,'!themis',exists=exists
if keyword_set(exists) and not keyword_set(reset) then return

themis_environment

themis= {   $
   themisstr, $
   local_data_dir:'', $
   remote_data_dir:'' $
}

defsysv,'!themis',themis

!themis.local_data_dir = getenv('THEMIS_DATA_DIR')
!themis.remote_data_dir= getenv('THEMIS_REMOTE_HOST_DIR')

if keyword_set(!themis.remote_data_dir) then begin
   if file_test(/write,!themis.local_data_dir) eq 0 then begin
       message,/info,'Local data directory is not writeable!. Disabling remote host.'
       !themis.remote_data_dir =''
   endif

endif



printdat,/values,!themis,'!themis'


cdf_lib_info,version=v,subincrement=si,release=r,increment=i,copyright=c
cdf_version = string(format="(i0,'.',i0,'.',i0,a)",v,r,i,si)

cdf_version_readmin = '3.1.0'
cdf_version_writemin = '3.1.1'

if cdf_version lt cdf_version_readmin then begin
   message,'Your version of the CDF library ('+cdf_version+') is unable to read THEMIS data files.',/info
   message,'Please go to the following url to learn how to patch your system:' ,/info
   message,'http://cdf.gsfc.nasa.gov/html/idl62_or_earlier_and_cdf3_problems.html',/info
   message,"You can have your data. You just can't read it! Sorry!"
endif

if cdf_version lt cdf_version_writemin then begin
   message,'Your version of the CDF library ('+cdf_version+') is unable to correctly write some CDF data files.',/info
   message,'Please go to the following url to learn how to patch your system:' ,/info
   message,'http://cdf.gsfc.nasa.gov/html/idl62_or_earlier_and_cdf3_problems.html',/info
endif


end

