;+
;NAME:
;thm_ui_check_dbase
;PURPOSE:
;Check for, and download files
;CALLING SEQUENCE:
;thm_ui_check_dbase, st_time, En_time, datatype, station, spacecraft,
;                    data_level
;INPUT:
;st_time, en_time = start and end times in seconds from
;                   1-jan-1970 0:00
;datatype, the type of data, a string, of form 'thg_gmag', for example
;station, the ground station of the gmag data
;spacecraft = the spacecraft_id string, or 'thg' for ground data
;data_level = 'L0','L1', or 'l2'
;HISTORY:
; 27-nov-2006, jmm, jimm@ssl.berkeley.edu
; 12-dec-2006, changed to call sock_copy, instead of
;              thm_ui_temp_sock_copy. changed datatype to dtpye to
;              avoid conflicts with ssw function
;
;$LastChangedBy J McTiernan $
;$LastChangedDate 12-Dec-2006 $
;$LastChangedRevision changed to call file_retrieve, then sock_copy$
;$URL mailto:jimm@ssl.berkeley.edu $
;
;-
Pro thm_ui_check_dbase, st_time, en_time, dtype0, station0, $
                        spacecraft0, data_level0, files_out, $
                        auto_download = auto_download, $
                        _extra = _extra

  themis_init                   ;just in case...
  dtype = strlowcase(strcompress(dtype0, /remove_all))
  station = strlowcase(strtrim(station0, 2))
  spacecraft = strlowcase(strtrim(spacecraft0, 2))
  data_level = strlowcase(strtrim(data_level0, 2))

  ref_time = time_double('2005-01-01/00:00:00')
  one_day = 24.0d0*3600.0d0
  st_day = long(st_time-ref_time)/one_day
  en_day = long(en_time-ref_time)/one_day
  ndays = en_day-st_day
  If(ndays Gt 0) Then Begin
    en_0_test = (en_time-ref_time) Mod one_day ;if this is zero, drop a day
    If(en_0_test Eq 0) Then ndays = ndays-1
  Endif
  file_test_time = st_time+one_day*dindgen(ndays+1)
;thg_mag data and asi data can have multiple stations
  If(dtype Eq 'mag') Then Begin
    If(station Eq 'all') Then Begin
      stations = thm_ui_mag_stationlist()
      stations = stations[1:*]
    Endif Else stations = strsplit(station, ' ', /extract)
    nstat = n_elements(stations)
    sc = 'thg'
    nsc = 1
  Endif Else If(dtype Eq 'asi') Then Begin
    If(station Eq 'all') Then Begin
      stations = thm_ui_asi_stationlist()
      stations = stations[1:*]
    Endif Else stations = strsplit(station, ' ', /extract)
    nstat = n_elements(stations)
    sc = 'thg'
    nsc = 1
  Endif Else Begin
    sc = spacecraft
    nsc = n_elements(spacecraft)
    nstat = 1
    stations = 'XXXX'
  Endelse
;Build up an array of filenames
  count = 0l
  For i = 0, nsc-1 Do For j = 0, nstat-1 Do Begin
    fj = thm_mk_filename(file_test_time, sc = sc[i], level = data_level, $
                         data_type = dtype, station = stations[j], $
                         version = 1)
    If(is_string(fj)) Then Begin
      If(count Eq 0) Then files_in = temporary(fj) $
      Else files_in = [temporary(files_in), temporary(fj)]
      count = n_elements(files_in)
    Endif
  Endfor
  If(count Eq 0) Then Begin
    message, /info, 'No Files?'
    files_out = -1
  Endif Else Begin
    n = n_elements(files_in)
    files_out = strarr(n)
;for each file, see if it exists, and see if it's on the server
    test_dir = strarr(n)
    For j = 0, n-1 Do test_dir[j] = thm_mk_data_dir(files_in[j], /name_only)
    files_out = test_dir+files_in
    ok = file_test(files_out)
;If there are missing files, prompt for downloading
    not_here = where(ok Eq 0, nnot_here)
    If(nnot_here Gt 0) Then Begin
;test to see if the files exist on the server
;the server and directories need to be split for sock_find
      If(is_string(!themis.remote_data_dir)) Then Begin
        xxxl = strsplit(!themis.remote_data_dir, '/')
;The first two are http:, and the server name, the rest is the data_dir
        only_data_dir = strmid(!themis.remote_data_dir, xxxl[2])
        server = strmid(!themis.remote_data_dir, 0, xxxl[2])
        sock_ping, server, status
        file_count = 0l
        If(status Eq 1) Then Begin
          For j = 0, nnot_here-1 Do Begin
            f = files_in[not_here[j]]
            test_dir = thm_mk_data_dir(f,  themis_data_dir = only_data_dir)
            fff = concat_dir(test_dir, f)
            v = sock_find(server, fff)
            If(is_string(v)) Then Begin
              If(file_count Eq 0) Then f_on_server = temporary(v) $
              Else f_on_server = [temporary(f_on_server), temporary(v)]
              file_count = file_count+1l
            Endif
          Endfor
        Endif
;If there are any files left, do the download
        If(file_count Gt 0) Then Begin
          If(keyword_set(auto_download)) Then yn = 1b Else Begin
            yn = yesno_widget_fn('Download Data files?', list = f_on_server)
          Endelse
          If(yn) Then Begin
            For j = 0, file_count-1 Do Begin
              f = file_basename(f_on_server[j])
              message, /info, 'Downloading: '+f
              remote_file = concat_dir(server, f_on_server[j])
              local_dir = thm_mk_data_dir(f)
              local_file = concat_dir(local_dir, f)
              print, 'Copying Remote_file=', remote_file
              print, 'To Local file=', local_file
              sock_copy, remote_file, out_dir = local_dir
              ok_load = file_test(local_file)
              If(ok_load Ne 1) Then message, /info, 'File: '+f+' Not downloaded'
            Endfor
          Endif Else message, /info, 'Files not downloaded'
        Endif
      Endif
    Endif
  Endelse
  Return
End

  
